//
// Brick Fragment shader:
// Example from
// http://www.awprofessional.com/articles/article.asp?p=171029&seqNum=1
//
// A Simple OpenGL Shading Example By Randi Rost.
// Sample Chapter is provided courtesy of Addison Wesley Professional
// From the Book  OpenGL Shading Language.
//

// Flag to tell if there is or not Video input
//
uniform float Video;

// Flag to tell fragment shader to render brick pattern or not.
//
uniform float RenderBrickPattern;

// Toggle among different teapot shaders.
//
uniform int SwitchShaders;

// Texture 
uniform sampler2D TexLoc;
const vec3 Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3 Yunitvec = vec3 (0.0, 1.0, 0.0);

uniform vec3 BaseColor;
uniform float MixRatio;

// We're perfectly happy using the depth value that's been computed by the OpenGL rasterization stage. 
// Therefore, the entire purpose of this shader is to compute the color of the current fragment. 
//

// The brick fragment shader uses the parameters that are illustrated in Figure 6.1. of OpenGL Shading Language book.
// These are defined as uniform variables as follows: 
//
// Parameters that control the brick color, 
// mortar color, 
// brick-to-brick horizontal distance, 
// brick-to-brick vertical distance, 
// brick width fraction (ratio of the width of a brick to the overall horizontal distance between two adjacent bricks), 
// and brick height fraction (ratio of the height of a brick to the overall vertical distance between two adjacent bricks).
//
// Brick size and brick percentage parameters will both be stored in user-defined uniform variables of type vec2. 
// The horizontal distance between two bricks, including the width of the mortar, will be provided by BrickSize.x. 
// The vertical distance between two rows of bricks, including the height of the mortar, will be provided by BrickSize.y. 
// These two values will be given in units of modeling coordinates. The fraction of BrickSize.x represented by the brick 
// only will be provided by BrickPct.x. The fraction of BrickSize.y represented by the brick only will be provided by BrickPct.y. 
// These two values will be in the range [0,1]. Finally, the brick color and the mortar color will be represented by the 
// variables BrickColor and MortarColor. 
//
// The values of BrickPct.x and BrickPct.y can be computed by the application to give a uniform mortar width in both 
// directions based on the ratio of column width to row height, or they can be chosen arbitrarily to give a mortar 
// appearance that looks right. 
//
uniform vec3  BrickColor, MortarColor;
uniform vec3  BrickSize;
uniform vec3  BrickPct;

const float brickHeight = BrickSize.y * BrickPct.y;

// Note that both of the varying variables we defined in our fragment shader are defined with the same type that was used to 
// define them in our vertex shader. A link error would be generated if this were not the case. 
// Further explanation of these two variables can be found in brick.vert
//

// We want our brick pattern to be applied in a consistent way to our geometry in order to have the object look the same no 
// matter where it is placed in the scene or how it is rotated. The key to determining the placement of the brick pattern is 
// the modeling coordinate position that is computed by the vertex shader and passed in the varying variable MCposition: 
//
varying vec3  MCposition;// This variable was computed at each vertex by the vertex shader in the previous section, and it 
						 // is interpolated across the primitive and made available to the fragment shader at each fragment 
						 // location. Our fragment shader can use this information to determine where the fragment location 
						 // is in relation to the algorithmically defined brick pattern. 

varying float LightIntensity; // This varying variable contains the interpolated value for the light intensity that we computed at 
							  // each vertex in our vertex shader. 

// For texture with Environment Mapping
varying vec3 Normal;
varying vec3 EyeDir;

// Gauss
const vec3 sigma = vec3(0.005, 0.005, 0.005);

void main(void)
{  
	// define all our local variables before they're used at the beginning of our main function.
    vec3  color;
    vec3  position, useBrick;

	// Don't waste time if I don't want to render brick pattern..
	if( RenderBrickPattern == 0.0 )
	{
		if( SwitchShaders == 1 )
		{	
			if (LightIntensity > 0.80)
				color = vec3(0.8,0.8,0.8);
			else 
			if (LightIntensity > 0.5)
				color = vec3(0.4,0.4,0.8);	
			else if (LightIntensity > 0.25)
				color = vec3(0.2,0.2,0.4);
			else
				color = vec3(0.1,0.1,0.1);		
				
			gl_FragColor = vec4 (color, 1.0);
		}

		if( SwitchShaders == 2 )
		{
			// Simeple texture
			vec3 texColor = vec3 (texture2D(TexLoc, gl_TexCoord[0].st) );	
			gl_FragColor  = vec4 (texColor * LightIntensity, 1.0);
		}
		
		if( SwitchShaders == 3 )
		{
			// compute reflection vector
			vec3 reflectDir = reflect(EyeDir, Normal);

			// compute altitude and azimuth angles
			vec2 index;

			index.y = dot(normalize(reflectDir), Yunitvec);
			reflectDir.y = 0.0;
			index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;


			// Translate index values into proper range
			if (reflectDir.z >= 0.0)
				index = (index + 1.0) * 0.5;
			else
			{
				index.t = (index.t + 1.0) * 0.5;
				index.s = (-index.s) * 0.5 + 1.0;
			}

			// if reflectDir.z >= 0.0, s will go from 0.25 to 0.75
			// if reflectDir.z >= 0.0, s will go from 0.75 to 1.25
			// which is OK since we are wrapping

			// Do a lookup into the environment map
			vec3 envColor = vec3 (texture2D(TexLoc, index));

			// Add lighting to base colour and mix
			vec3 base = LightIntensity * BaseColor;
			envColor = mix(envColor, base, MixRatio);

			gl_FragColor = vec4(envColor, 1.0);
		}				
	}
	else
	{
    // This statement divides the fragment's x position in modeling coordinates by the column width and the y position in modeling 
    // coordinates by the row height. This gives us a "brick row number" (position.y) and a "brick number" within that row 
    // (position.x). Keep in mind that these are signed, floating-point values, so it is perfectly reasonable to have negative 
    // row and brick numbers as a result of this computation.
    //		
	position = (MCposition)/ BrickSize;

	// Next, we'll use a conditional to determine whether the fragment is in a row of bricks that is offset: 
	// My approach is a little bit different thn the book..The book's approach gave me some inconsistancies between rows..
	// Don't know why
	//
	if( int(position.y+0.1)%2 != 0 )
	{
		position.y = MCposition.y;
		position.x = 2.0 - MCposition.x; // Since I carefully set the brick dimensions, to obtain the similar 'offset' pattern
		position.z = 2.0 - MCposition.z; // like the book, I just scan 0 to 2 in one (even) row, and the next one (odd) from 2 to 0.
		
		position = (position)/ BrickSize;// Then, realculate MCposition and continue.
	}
		
//  if( fract(position.y * 0.5) > 0.5) // The "brick row number" (position.y) is multiplied by 0.5, and the result is compared 
//  {								   // against 0.5. Half the time (or every other row) this comparison will be true, and the 	
//      position.x += 0.5;			   // "brick number" value (position.x) is incremented by 0.5 to offset the entire row by half 
//		position.z += 0.5;			   // the width of a brick.	
//	}								   // Remember: frac is always positive in the range of [0.0, 1.0]

	// We need to compute the fragment's location within the current brick: 
	//
	position = fract(position); // This computation provides us with the vertical and horizontal position within a 
							    // single brick. This will be used as the basis for determining whether to use the 
								// brick color or the mortar color. 

	// We need a function that gives us a value of 1.0 when the brick color should be used and 0 when the mortar color 
	// should be used. If we can achieve this, we'll end up with a simple way to choose the appropriate color: 
	// The step function can be used to achieve the desired effect. It takes two arguments, an edge (or threshold) 
	// and a parameter, to test against that edge. If the value of the parameter to be tested is less than or equal 
	// to the edge value, the function returns 0; otherwise, it returns 1.0. 
	// In typical use, the step function is used to produce a pattern of pulses (i.e., a square wave) where the function 
	// starts off at 0 and rises to 1 when the threshold is reached. We can get a function that starts off at 1.0 and 
	// drops to 0 just by reversing the order of the two arguments provided to this function: 
	//
	// DON'T USE STEP!
    // useBrick = step(position, BrickPct); // In these two lines, we compute two values that tell us whether we are in the brick 
										 // or in the mortar in the horizontal direction (useBrick.x) and in the vertical direction 
										 // (useBrick.y). 
										 // The built-in function step will produce a value of 0 when BrickPct.x <= position.x and 
										 // a value of 1.0 when BrickPct.x > position.x. Because of the fract function, we know that 
										 // position.x will vary from [0,1). The variable BrickPct is a uniform variable, so its 
										 // value will be constant across the primitive. This means that the value of useBrick.x 
										 // will be 1.0 when the brick color should be used and 0 when the mortar color should be 
										 // used as we move horizontally. The same thing is done in the vertical direction using 
										 // position.y and BrickPct.y to compute the value for useBrick.y. By multiplying useBrick.x 
										 // and useBrick.y together, we can get a value of 0 or 1.0 that will let us select the 
										 // appropriate color for the fragment. 

	// useBrick = smoothstep(position, 1.0, BrickPct);
	
	// Since smoothstep doesn't do the trick in one of the edges edges, I use a gaussian function 
	// (with the bell inverted) with the mean center in the first edge (from brick to molder).
	// Work like a charm!! And better than the cubic use by smoothstep.
	// 
	vec3 temp;
	
	temp = -pow((position - BrickPct), vec3(2.0, 2.0, 2.0))/sigma;
	temp = 1.0 - exp( temp );
	useBrick = temp;
				
	// All that remains is to compute our final color value and store it in the special variable gl_FragColor: 
	// Here we compute the color of the fragment and store it in the local variable color. The built-in function mix is used to 
	// choose the brick color or the mortar color, depending on the value of useBrick.x * useBrick.y. 
	// Because useBrick.x and useBrick.y can have values of only 0 (mortar) or 1.0 (brick), we will choose the brick color only 
	// if both values are 1.0; otherwise, we will choose the mortar color. 
	// The resulting value is then multiplied by the light intensity, and that result is stored in the local variable color. 
	// This local variable is a vec3, and gl_FragColor is defined as a vec4, so we create our final color value by using a 
	// constructor to add a fourth component (alpha) equal to 1.0 and assign the result to the built-in variable gl_FragColor. 
	//

	// Set flat texture (mortar)at top and bottom and top/bottom row.
	// Calculate first (look at const var) the actual brick height (without mortar),
	// then check if it is the first or last row and paint it with mortar color.
	//

	if( (MCposition.y < brickHeight) || (MCposition.y > (1.98 - brickHeight)))
		useBrick.y = 0.0;
	
	if( SwitchShaders == 3 )
	{
		float alpha;
		
		// compute reflection vector
		vec3 reflectDir = reflect(EyeDir, Normal);

		// compute altitude and azimuth angles
		vec2 index;

		index.y = dot(normalize(reflectDir), Yunitvec);
		reflectDir.y = 0.0;
		index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;


		// Translate index values into proper range
		if (reflectDir.z >= 0.0)
			index = (index + 1.0) * 0.5;
		else
		{
			index.t = (index.t + 1.0) * 0.5;
			index.s = (-index.s) * 0.5 + 1.0;
		}

		// if reflectDir.z >= 0.0, s will go from 0.25 to 0.75
		// if reflectDir.z >= 0.0, s will go from 0.75 to 1.25
		// which is OK since we are wrapping

		// Do a lookup into the environment map
		vec3 envColor = vec3 (texture2D(TexLoc, index));

		// Add lighting to base colour and mix
		vec3 base = LightIntensity * BaseColor;
		envColor = mix(envColor, base, MixRatio);

	    color  = mix(MortarColor * LightIntensity, envColor, useBrick.x * useBrick.y * useBrick.z );

		gl_FragColor = vec4(color, 1.0);
	}
	else
	{
	    color  = mix(MortarColor, BrickColor, useBrick.x * useBrick.y * useBrick.z );
	    color *= LightIntensity;
		gl_FragColor = vec4 (color, 1.0);
    }
	}
}