//=============================================================================
//
// PGRFlyCaptureTest.cpp
//
// The PGRFlyCaptureTest sample program is a simple program designed to report 
// information related to all compatible cameras attached to the host system, 
// capture a series of images from a single camera, record the amount of time 
// taken to grab these images, then save the last image in the current 
// directory.
//
// The program first creates two FlyCapture variables, one of which is a
// FlyCaptureError variable. In order to reliably debug your application, 
// define a variable of this type to capture meaningful errors returned by API 
// functions. The program then enumerates, or lists, all PGR cameras sitting 
// on the bus and their bus index, starting at zero. This function does not 
// enumerate non-PGR cameras.
//
// A FlyCaptureContext is created, which acts as a handle to the camera, and 
// is required to initialize and start the camera. The program then initializes 
// the camera located at bus index zero and associates it with the camera 
// context. Multiple cameras connected to the FireWire bus are enumerated at 
// bus indices (FireWire nodes) that start at 0. Information related to the 
// initialized camera (model, serial number, DCAM compliance, etc.) is then 
// reported to the user.
// 
// The initialized camera is then started at the default resolution and image 
// and frame rate of the camera. This function also (silently) allocates four 
// buffers in main memory that are used to hold the images that are streamed in 
// from the camera. Once a camera has been started, it immediately begins 
// capturing and streaming images via Direct Memory Access (DMA) to these 
// memory buffers. Once these buffers are full, they will be overwritten with 
// consecutive images unless they are locked by the user.
// 
// The program then creates a FlyCaptureImage variable. The FlyCaptureImage 
// structure contains the image data, as well as video mode, whether the image 
// is stippled (color) and timestamp information.
// 
// In the grab loop, when a call to flycaptureGrabImage2() is made a pointer to 
// the image buffer (&image) with the newest (latest) complete image is returned. 
// The call to flycaptureGrabImage2() does not involve copying, so it is quite 
// fast. The user will never be given an image that is older than one that has 
// already been seen. Once the pointer to the buffer is returned to the user, 
// this buffer remains locked until flycaptureGrabImage2() is called again. If 
// no buffer contains an image newer than the last returned, then the 
// flycaptureGrabImage2() call will block until a new image is available.
// 
// After the program exits the grab loop, it converts the last image grabbed to a 
// 32-bit per pixel image that can be displayed by Microsoft Windows (which uses 
// the BGRU format). If the camera is not a color camera, Y8 and Y16 images are 
// converted to BGRU greyscale. It then saves the image to file using the PGM 
// file format.
// 
// Finally, the program stops the camera and destroys the camera context. In 
// order to prevent memory leaks from occurring, this function must be called 
// when the user is finished with the FlyCaptureContext.

//
//=============================================================================

//=============================================================================
// System Includes
//=============================================================================
#include <assert.h>
#include <stdio.h>
#include <sys/timeb.h>
#include <memory.h>

//=============================================================================
// Project Includes
//=============================================================================
#include <pgrflycapture.h>

//
// The maximum number of cameras on the bus.
//
#define _MAX_CAMS       32
//
// The index of the camera to grab from.
//
#define _CAMERA_INDEX   0

//
// What file format should we save the processed image as?
//
//#define SAVE_FORMAT     FLYCAPTURE_FILEFORMAT_PPM
//#define FILENAME_CONVERTED "converted.ppm"

#define SAVE_FORMAT     FLYCAPTURE_FILEFORMAT_BMP
#define FILENAME_CONVERTED "converted.bmp"
#define FILENAME_RAW       "raw.pgm"

// Global vers
static FlyCaptureImage   image;
static FlyCaptureContext context;  
static FlyCaptureError   error;
static bool startvideo = false;

//
// Helper code to handle a FlyCapture error.
//
#define _HANDLE_ERROR( error, function ) \
   if( error != FLYCAPTURE_OK ) \
   { \
      printf( "%s: %s\n", function, flycaptureErrorToString( error ) ); \
      return false; \
   } \

void
reportCameraInfo( const FlyCaptureInfoEx* pinfo )
{
   printf( "Serial number: %d\n", pinfo->SerialNumber );
   printf( "Camera model: %s\n", pinfo->pszModelName );
   printf( "Camera vendor: %s\n", pinfo->pszVendorName );
   printf( "Sensor: %s\n", pinfo->pszSensorInfo );
   printf( "DCAM compliance: %1.2f\n", (float)pinfo->iDCAMVer / 100.0 );
   printf( "Bus position: (%d,%d).\n", pinfo->iBusNum, pinfo->iNodeNum );
}


bool
initStartVideo( void )
{
	if( startvideo  )
		return( true );
   //
   // Enumerate the cameras on the bus.
   //
   FlyCaptureInfoEx  arInfo[ _MAX_CAMS ];
   unsigned int	     uiSize = _MAX_CAMS;

   error = flycaptureBusEnumerateCamerasEx( arInfo, &uiSize );
   _HANDLE_ERROR( error, "flycaptureBusEnumerateCameras()" );

   for( unsigned int uiBusIndex = 0; uiBusIndex < uiSize; uiBusIndex++ )
   {
      FlyCaptureInfoEx* pinfo = &arInfo[ uiBusIndex ];
      printf( 
         "Index %u: %s (%u)\n",
         uiBusIndex,
         pinfo->pszModelName,
         pinfo->SerialNumber );
   }

   //
   // Create the context.
   //
   error = flycaptureCreateContext( &context );
   _HANDLE_ERROR( error, "flycaptureCreateContext()" );
   
   //
   // Initialize the camera.
   //
   printf( "Initializing camera %u.\n", _CAMERA_INDEX );
   error = flycaptureInitialize( context, _CAMERA_INDEX );
   _HANDLE_ERROR( error, "flycaptureInitialize()" );

   //
   // Report camera info.
   //
   FlyCaptureInfoEx info;
   error = flycaptureGetCameraInfo( context, &info );
   _HANDLE_ERROR( error, "flycaptureGetCameraInfo()" );

   printf( "Camera info:\n" );
   reportCameraInfo( &info );

   //
   // Start grabbing images in the current videomode and framerate.
   //
   printf( "Starting camera.\n\n" );
   error = flycaptureStart( 
      context, 
      FLYCAPTURE_VIDEOMODE_ANY,
      FLYCAPTURE_FRAMERATE_ANY );
   _HANDLE_ERROR( error, "flycaptureStart()" );

   //
   // Time the grabbing of _IMAGES_TO_GRAB images.
   //
   memset( &image, 0x0, sizeof( FlyCaptureImage ) );

   return( startvideo = true );
}

bool grabImg( char **img, int *w, int *h )
{
	if( !startvideo )
		return( false );

	error = flycaptureGrabImage2( context, &image );
	_HANDLE_ERROR( error, "flycaptureGrabImage2()" );

	*w = image.iCols;
	*h = image.iRows;
	*img = (char *)image.pData;

	return( true );
}

bool stopGrab( void )
{
	if( !startvideo )
		return( false );

	startvideo = false;
   //
   // Stop the camera
   //
   error = flycaptureStop( context );
   _HANDLE_ERROR( error, "flycaptureStop()" );

   //
   // Destroy the context.
   //
   error = flycaptureDestroyContext( context );
   _HANDLE_ERROR( error, "flycaptureBusEnumerateCameras()" );

//   delete [] imageConverted.pData;
	
   printf("Camera has been stopped!\n");

   return( true );
}
